unit PDMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ShellCtrls, ExtCtrls;

{ Design By Howard Chen. howardchen@pie.com.tw }

type
  TArrayOfPoint = array of TPoint;

  TPolygonPointArray = class(TObject)
  private
    FList: TList;
    function GetCount: Integer;
    function GetItem(Index: Integer): TPoint;
    procedure SetItem(Index: Integer; const Value: TPoint);
  public
    constructor Create;
    destructor Destroy; override;

    function Add(Point: TPoint): Integer; overload;
    function Add(X, Y: Integer): Integer; overload;
    procedure Delete(Index: Integer);
    procedure Clear;

    function GetArray: TArrayOfPoint;
    function FindNear(Point: TPoint): Integer; overload;
    function FindNear(X, Y: Integer): Integer; overload;

    property Item[Index: Integer]: TPoint read GetItem write SetItem; default;
    property Count: Integer read GetCount;
  end;

type
  TForm1 = class(TForm)
    WorkPanel: TImage;
    Button1: TButton;
    Label1: TLabel;
    Label2: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure WorkPanelMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormDestroy(Sender: TObject);
    procedure WorkPanelMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure WorkPanelMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Button1Click(Sender: TObject);
  private
    FPolygonPointArray: TPolygonPointArray;
    FLastPoint: TPoint;
    FAddPointMode: Boolean;
    FDragPointMode: Boolean;
    FDrawBmp: TBitmap;
    FDragPointIndex: Integer;
    procedure DrawBmpToWorkPanel;
  public
  end;

var
  Form1: TForm1;


implementation

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
begin
  FPolygonPointArray := TPolygonPointArray.Create;
  FDrawBmp := TBitmap.Create;

  FDrawBmp.Width := WorkPanel.Width;
  FDrawBmp.Height := WorkPanel.Height;
  FDrawBmp.Canvas.Brush.Color := clWhite;
  FDrawBmp.Canvas.FillRect(Rect(0, 0, FDrawBmp.Width, FDrawBmp.Height));

  DrawBmpToWorkPanel;

  DoubleBuffered := True;
  FAddPointMode := False;
  FDragPointMode := False;
  FDragPointIndex := -1;
  Label1.Caption := '';
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  FPolygonPointArray.Free;
end;

procedure TForm1.DrawBmpToWorkPanel;
var
  i: Integer;
begin
  FDrawBmp.Canvas.Brush.Color := clWhite;
  FDrawBmp.Canvas.FillRect(Rect(0, 0, FDrawBmp.Width, FDrawBmp.Height));
  FDrawBmp.Canvas.Polygon(FPolygonPointArray.GetArray);

  if FDragPointMode then
  begin
    for i := 0 to FPolygonPointArray.Count-1 do
      FDrawBmp.Canvas.Rectangle(FPolygonPointArray[i].X-3, FPolygonPointArray[i].Y-3,
          FPolygonPointArray[i].X+3, FPolygonPointArray[i].Y+3);
  end;
  
  WorkPanel.Canvas.Draw(0, 0, FDrawBmp);
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  FDragPointMode := not FDragPointMode;

  DrawBmpToWorkPanel;

  if FDragPointMode then
    Label1.Caption := 'Drag Point Mode'
  else
    Label1.Caption := '';
end;

procedure TForm1.WorkPanelMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbLeft then
    if FDragPointMode then
      FDragPointIndex := FPolygonPointArray.FindNear(X, Y);
  Label2.Caption := IntToStr(FDragPointIndex);
end;

procedure TForm1.WorkPanelMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  if FDragPointMode then
  begin
    if FDragPointIndex >= 0 then
    begin
      FPolygonPointArray[FDragPointIndex] := Point(X, Y);
      Label2.Caption := IntToStr(FDragPointIndex);

      DrawBmpToWorkPanel;
    end;
  end
  else if FAddPointMode then
  begin
    DrawBmpToWorkPanel;
    WorkPanel.Canvas.MoveTo(FLastPoint.X, FLastPoint.Y);
    WorkPanel.Canvas.LineTo(X, Y);
  end;
end;

procedure TForm1.WorkPanelMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if FDragPointMode then
  begin
    FDragPointIndex := -1;
    Label2.Caption := IntToStr(FDragPointIndex);

    DrawBmpToWorkPanel;
  end
  else
  begin
    if Button = mbLeft then
    begin
      if not FAddPointMode then
        FPolygonPointArray.Clear;

      FLastPoint.X := X;
      FLastPoint.Y := Y;

      FAddPointMode := True;
      FPolygonPointArray.Add(X, Y);

      DrawBmpToWorkPanel;
    end
    else if Button = mbRight then
    begin
      FAddPointMode := False;

      DrawBmpToWorkPanel;
    end;
  end;
end;

{ TPolygonPointArray }

constructor TPolygonPointArray.Create;
begin
  FList := TList.Create;
end;

destructor TPolygonPointArray.Destroy;
begin
  Clear;
  FList.Free;

  inherited Destroy;
end;

function TPolygonPointArray.Add(Point: TPoint): Integer;
var
  PP: PPoint;
begin
  New(PP);
  Move(Point, PP^, SizeOf(TPoint));
  Result := FList.Add(PP);
end;

function TPolygonPointArray.Add(X, Y: Integer): Integer;
begin
  Result := Add(Point(X, Y));
end;

procedure TPolygonPointArray.Delete(Index: Integer);
begin
  Dispose(PPoint(FList[Index]));
  FList.Delete(Index);
end;

procedure TPolygonPointArray.Clear;
var
  i: Integer;
begin
  for i := Count-1 downto 0 do
    Delete(i);
end;

function TPolygonPointArray.GetArray: TArrayOfPoint;
var
  Ret: TArrayOfPoint;
  i: Integer;
begin
  SetLength(Ret, Count);

  for i := 0 to Count-1 do
    Ret[i] := Item[i];

  Result := Ret;
end;

function TPolygonPointArray.FindNear(Point: TPoint): Integer;
begin
  Result := FindNear(Point.X, Point.Y);
end;

function TPolygonPointArray.FindNear(X, Y: Integer): Integer;
var
  i: Integer;
begin
  Result := -1;
  
  for i := Count-1 downto 0 do
    if (Abs(PPoint(FList[i]).X - X) < 3) and (Abs(PPoint(FList[i]).Y - Y) < 3) then
    begin
      Result := i;
      Break;
    end;
end;

function TPolygonPointArray.GetCount: Integer;
begin
  Result := FList.Count;
end;

function TPolygonPointArray.GetItem(Index: Integer): TPoint;
begin
  Result := PPoint(FList[Index])^;
end;

procedure TPolygonPointArray.SetItem(Index: Integer; const Value: TPoint);
begin
  Move(Value, PPoint(FList[Index])^, SizeOf(TPoint));
end;

end.
